# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Configuration du jeu réseau et connexion.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import bge

import utils, input_controller



# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = None
scene = None



# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postDrawTexts():
    """
    affichage des textes.
    """
    utils.postDrawTexts('lan_loop')

def doAction(what):
    """
    lance l'action correspondant au menu qui a été cliqué.
    """
    if what == 92:
        # connexion :
        testConnexion()
    elif what == 91:
        # retour au menu :
        bge.logic.globalDict['GAME']['state'] = 'lan_quit'
        utils.saveSettings()
        scene.replace('menu')
    elif what == 90:
        # lancer le jeu :
        bge.logic.globalDict['GAME']['state'] = 'lan_quit'
        utils.saveSettings()
        theScene = 'game'
        gameName = bge.logic.globalDict['MENU']['gameName']
        if bge.logic.globalDict['GAMES'][gameName] != []:
            theScene = 'levels'
        scene.replace(theScene)

def testConnexion():
    import lan_client
    lang = bge.logic.globalDict['MENU']['lang'] 
    host = bge.logic.globalDict['TEXTS']['InputServer'][0]
    user = bge.logic.globalDict['TEXTS']['InputUser'][0]
    """ *************Password*************
    password = bge.logic.globalDict['TEXTS']['InputPassword'][0]
    """
    password = ''
    # si user est vide, on refuse la connexion :
    if user == '':
        text = utils.tr('other', 'Failure', lang)
        utils.setText('State', text=text)
        text = utils.tr('other', 'You need a user name', lang)
        utils.setText('message', text=text)
        return
    # test de connexion au serveur :
    ok, others = lan_client.testConnexion(host, utils.LAN_PORT)
    if not(ok):
        text = utils.tr('other', 'Failure', lang)
        utils.setText('State', text=text)
        text = utils.tr('other', 'Server address not valid|(or not launched)', lang)
        utils.setText('message', text=text)
        return
    # on vérifie si le user n'est pas déjà pris :
    if user in others:
        text = utils.tr('other', 'Failure', lang)
        utils.setText('State', text=text)
        text = utils.tr('other', 'This username is already taken', lang)
        utils.setText('message', text=text)
        return
    # donc c'est bon :
    bge.logic.globalDict['LAN']['host'] = host
    bge.logic.globalDict['LAN']['user'] = user
    bge.logic.globalDict['LAN']['password'] = password
    text = utils.tr('other', 'OK', lang)
    utils.setText('State', text=text)
    text = utils.tr('other', 'You can play', lang)
    utils.setText('message', text=text)
    menuItem = bge.logic.globalDict['TEMP']['menuItems']['item_90']
    menuItem['ok'] = True
    utils.setText('item_90', color=utils.colors['orange'])

def keyboardManagement():
    """
    gestion du clavier
    """
    keyboard = bge.logic.keyboard.events
    textName = bge.logic.globalDict['TEMP']['actualInput']
    text = bge.logic.globalDict['TEXTS'][textName][0]
    lastKey = None
    for key in keyboard:
        if keyboard[key] == input_controller.JUST_ACTIVATED:
            lastKey = key
    if lastKey != None:
        # touche SHIFT :
        shift = False
        if keyboard[input_controller.LEFT_SHIFT_KEY] == input_controller.ACTIVE:
            shift = True
        elif keyboard[input_controller.RIGHT_SHIFT_KEY] == input_controller.ACTIVE:
            shift = True
        if bge.logic.globalDict['TEMP']['CAPSLOCK']:
            shift = not(shift)
        # cas spéciaux : CAPSLOCK, ENTER, TAB, BACKSPACE :
        if lastKey == input_controller.BACKSPACE_KEY:
            text = text[:-1]
        elif lastKey in (input_controller.ENTER_KEY, input_controller.TAB_KEY):
            # on passe au input et à la zone suivants :
            actualInput = bge.logic.globalDict['TEMP']['actualInput']
            """ *************Password*************
            inputs = ('InputServer', 'InputUser', 'InputPassword')
            """
            inputs = ('InputServer', 'InputUser')
            actualInput = utils.nextInList(inputs, actualInput)
            text = bge.logic.globalDict['TEXTS'][actualInput][0]
            """ *************Password*************
            inputs = {'InputServer':    'ZoneServer', 
                      'InputUser':      'ZoneUser', 
                      'InputPassword':  'ZonePassword'}
            """
            inputs = {'InputServer':    'ZoneServer', 
                      'InputUser':      'ZoneUser'}
            for element in inputs:
                if element == actualInput:
                    scene.objects[inputs[element]].visible = True
                else:
                    scene.objects[inputs[element]].visible = False
            bge.logic.globalDict['TEMP']['actualInput'] = actualInput
        elif lastKey == input_controller.CAPSLOCK_KEY:
            bge.logic.globalDict['TEMP']['CAPSLOCK'] = not(bge.logic.globalDict['TEMP']['CAPSLOCK'])
        else:
            text += bge.events.EventToCharacter(lastKey, shift)
        utils.setText(
            bge.logic.globalDict['TEMP']['actualInput'], 
            text, 
            color=utils.colors['white'], 
            width=2)



# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    state = bge.logic.globalDict['GAME'].get('state', '')
    if not(state in ('lan_first', 'lan_loop', 'lan_quit')):
        bge.logic.globalDict['GAME']['state'] = 'lan_first'

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] == 'lan_first':
        # valeurs initiales (lues depuis le fichier de config si existe) :
        utils.loadSettings()
        lang = bge.logic.globalDict['MENU']['lang']
        global controller, scene
        controller = bge.logic.getCurrentController()
        scene = bge.logic.getCurrentScene()
        # affichage des textes :
        scene.post_draw += [postDrawTexts]
        bge.logic.globalDict['TEXTS'] = {}
        # le titre :
        utils.setText(
            'progName', text=utils.PROGNAME,
            color=utils.colors['orange'], width=2, position=(0.1, 0.15))
        text = utils.tr('other', 'LanConfig', lang)
        utils.setText(
            'title', text=text,
            color=utils.colors['white'], width=3, position=(0.3, 0.15))
        # adresse du serveur :
        text = '{0} :'.format(utils.tr('other', 'Server', lang))
        utils.setText(
            'LabelServer', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.33))
        host = bge.logic.globalDict['LAN']['host']
        utils.setText(
            'InputServer', text=host, 
            color=utils.colors['white'], width=2, position=(0.5, 0.33))
        # utilisateur :
        text = '{0} :'.format(utils.tr('other', 'User', lang))
        utils.setText(
            'LabelUser', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.45))
        user = bge.logic.globalDict['LAN']['user']
        utils.setText(
            'InputUser', text=user,
            color=utils.colors['white'], width=2, position=(0.5, 0.45))
        # mot de passe :
        """
        text = '{0} :'.format(utils.tr('other', 'Password', lang))
        utils.setText('LabelPassword', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.57))
        utils.setText('InputPassword',
            color=utils.colors['white'], width=2, position=(0.5, 0.57))
        """
        # zone d'édition actuelle :
        bge.logic.globalDict['TEMP']['actualInput'] = 'InputServer'
        scene.objects['ZoneServer'].visible = True
        # état de la connexion :
        utils.setText(
            'State', text='', 
            color=utils.colors['white'], width=3, position=(0.8, 0.68))
        utils.setText(
            'message', text='', 
            color=utils.colors['white'], width=2, position=(0.3, 0.78))

        # création du menu :
        bge.logic.globalDict['TEMP']['menuItems'] = {}
        bge.logic.globalDict['TEMP']['menuOver'] = ''
        # connexion :
        utils.createMenuItem(
            'item_92', 
            name='CONNEXION', text=utils.tr('other', 'CONNECT', lang),
            position=(0.2, 0.68), what=92)
        # menu retour :
        utils.createMenuItem(
            'item_91', 
            name='MENU', text=utils.tr('menu', 'MENU', lang),
            position=(0.35, 0.95), what=91)
        # menu lancer le jeu :
        utils.createMenuItem(
            'item_90', 
            name='PLAY', text=utils.tr('menu', 'PLAY', lang),
            position=(0.65, 0.95), what=90, ok=False)

        if not('CAPSLOCK' in bge.logic.globalDict['TEMP']):
            bge.logic.globalDict['TEMP']['CAPSLOCK'] = False
        # on affiche la souris :
        bge.logic.mouse.position = bge.logic.globalDict['GAME']['cursorPosition']
        bge.render.showMouse(True)
        # initialisation terminée :
        bge.logic.globalDict['GAME']['state'] = 'lan_loop'

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] != 'lan_loop':
        return
    keyboardManagement()
    # récupération des coordonnées de la souris :
    cursorPosition = input_controller.getCursorPosition()
    # affichage des menus et récupération de l'action active :
    what = -1
    for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
        menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        if menuItem['ok']:
            menuPosition = menuItem['position']
            over = input_controller.mouseOver(cursorPosition, menuPosition)
            if over:
                what = menuItem['what']
                lang = bge.logic.globalDict['MENU']['lang']
                if menuItemName != bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = menuItemName
                    menuItem['width'] = menuItem['initialWidth'] * 1.5
                    utils.setText(menuItemName, width=menuItem['width'])
                    utils.playSound('over')
            else:
                menuItem['width'] = menuItem['initialWidth']
                utils.setText(menuItemName, width=menuItem['width'])
                if menuItemName == bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = ''
    inputState = input_controller.inputManagement('menu')
    # pour faire un screencast :
    if inputState['SCREENSHOT']:
        utils.makeScreenshot()
    # gestion des menus et autres actions :
    if (what > -1) and (inputState['ENTER']):
        utils.playSound('clic')
        doAction(what)
        return
    elif inputState['QUIT']:
        doAction(91)
        return
    # gestion du changement de focus par clic :
    try:
        actualZone = controller.sensors['Mouse'].hitObject.name
        zones = {'ZoneServer': 'InputServer', 'ZoneUser': 'InputUser'}
        if actualZone in zones:
            if inputState['ENTER']:
                bge.logic.globalDict['TEMP']['actualInput'] = zones[actualZone]
                scene.objects[actualZone].visible = True
                if actualZone == 'ZoneServer':
                    scene.objects['ZoneUser'].visible = False
                elif actualZone == 'ZoneUser':
                    scene.objects['ZoneServer'].visible = False
    except:
        pass


