# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Configuration du serveur pour un jeu réseau.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import sys
import socket
import subprocess
import time
import random

import bge

import utils, input_controller
import lan_client


# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = None
scene = None



# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postDrawTexts():
    """
    affichage des textes.
    """
    utils.postDrawTexts('server_loop')

def doAction(what):
    """
    lance l'action correspondant au menu qui a été cliqué.
    """
    if what == 81:
        # démarrer le serveur :
        changeServerState(True)
    elif what == 82:
        # arrêter le serveur
        changeServerState(False)
    elif what == 91:
        # retour au menu :
        bge.logic.globalDict['GAME']['state'] = 'server_quit'
        utils.clearGlobalDict()
        scene.replace('menu')
    elif what == 21:
        # changement de météo :
        lang = bge.logic.globalDict['MENU']['lang'] 
        weatherLang = bge.logic.globalDict['TEXTS']['item_21'][0]
        for w in utils.weatherDict['LIST']:
            if utils.tr('GAME_LAN', w, lang) == weatherLang:
                weather = w
        weather = utils.nextInList(utils.weatherDict['LIST'], weather)
        weatherLang = utils.tr('GAME_LAN', weather, lang)
        utils.setText('item_21', text=weatherLang)
    elif what == 22:
        # envoi de la météo :
        lang = bge.logic.globalDict['MENU']['lang'] 
        weatherLang = bge.logic.globalDict['TEXTS']['item_21'][0]
        for w in utils.weatherDict['LIST']:
            if utils.tr('GAME_LAN', w, lang) == weatherLang:
                weather = w
        sendMessage('WEATHERø{0}'.format(weather))
        bge.logic.globalDict['TEMP']['WEATHER'] = weather
    elif what == 31:
        # changement de jeu :
        lang = bge.logic.globalDict['MENU']['lang']
        gameLang = bge.logic.globalDict['TEXTS']['item_31'][0]
        for g in utils.lanGames:
            if utils.tr('GAME_LAN', g, lang) == gameLang:
                game = g
        newGame = utils.nextInList(utils.lanGames, game)
        newGameLang = utils.tr('GAME_LAN', newGame, lang)
        utils.setText('item_31', text=newGameLang)
    elif what == 32:
        # envoi du jeu :
        lang = bge.logic.globalDict['MENU']['lang']
        gameLang = bge.logic.globalDict['TEXTS']['item_31'][0]
        for g in utils.lanGames:
            if utils.tr('GAME_LAN', g, lang) == gameLang:
                game = g
        initGame(game)
    elif what == 41:
        # écriture et envoi d'un message :
        messageText = bge.logic.globalDict['TEMP']['SERVER_MESSAGE']
        bge.logic.globalDict['TEMP']['SERVER_MESSAGE'] = utils.doMessage(messageText)
    elif what == 42:
        # envoi du message :
        messageText = bge.logic.globalDict['TEMP']['SERVER_MESSAGE']
        if messageText != '':
            sendMessage('MESSAGEø{0}'.format(messageText))

def findMyIp():
    """
    récupère l'IP du poste
    """
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('google.com', 80))
        myIp = s.getsockname()[0]
        s.close()
    except:
        myIp = 'localhost'
    return myIp

def changeServerState(doRun=True):
    lang = bge.logic.globalDict['MENU']['lang'] 
    if doRun:
        if 'LANServer' in bge.logic.globalDict['TEMP']:
            return
        args = []
        bundledPython = bge.logic.globalDict['OTHER']['bundledPython']
        if bundledPython == '':
            return

        # on lance le fichier lan_server.py :
        root = bge.logic.expandPath('//')
        processFile = '{0}..{1}scripts{1}lan_server.py'.format(root, utils.SEP)
        processFile = bge.logic.expandPath(processFile)
        args = [bundledPython, processFile]
        """
        if sys.platform.startswith('win'):
            # vu que sous windows Python n'est pas forcément installé,
            # j'ai dû faire un exe :
            root = bge.logic.expandPath('//')
            processFile = '{0}..{1}scripts{1}winexe{1}lan_server.exe'.format(root, utils.SEP)
            processFile = bge.logic.expandPath(processFile)
            args = [processFile]
        else:
            # on lance le fichier lan_server.py :
            root = bge.logic.expandPath('//')
            processFile = '{0}..{1}scripts{1}lan_server.py'.format(root, utils.SEP)
            processFile = bge.logic.expandPath(processFile)
            args = [bundledPython, processFile]
        """
        host = bge.logic.globalDict['LAN']['host']
        args.append('HOST={0}'.format(host))
        try:
            process = subprocess.Popen(args)
        except:
            print('SERVER EXECUTION FAILED')
            return
        bge.logic.globalDict['TEMP']['LANServer'] = process
        # ICI
        time.sleep(2)
        ok = lan_client.main(host, utils.LAN_PORT)
        sendMessage('CONNECTøSERVERøSERVER')
        text = utils.tr('other', 'ServerIsRuning', lang)
    else:
        sendMessage('END')
        time.sleep(2.5)
        process = bge.logic.globalDict['TEMP']['LANServer']
        if process.poll() == None:
            process.terminate()
            time.sleep(1)
        del process
        del bge.logic.globalDict['TEMP']['LANServer']
        text = utils.tr('other', 'ServerIsStopped', lang)

    utils.setText('serverState', text=text)
    bge.logic.globalDict['TEMP']['menuItems']['item_91']['ok'] = not(doRun)
    bge.logic.globalDict['TEMP']['menuItems']['item_81']['ok'] = not(doRun)
    bge.logic.globalDict['TEMP']['menuItems']['item_82']['ok'] = doRun
    for menuItem in ('item_91', 'item_81', 'item_82'):
        if bge.logic.globalDict['TEMP']['menuItems'][menuItem]['ok']:
            utils.setText(menuItem, color=utils.colors['orange'])
        else:
            utils.setText(
                menuItem, 
                color=utils.colors['gray'], 
                width=bge.logic.globalDict['TEMP']['menuItems'][menuItem]['initialWidth'])

def sendMessage(message='', force=False):
    if 'LANServer' in bge.logic.globalDict['TEMP']:
        if (message != bge.logic.globalDict['LAN']['lastMessages'][0]) or (force):
            bge.logic.globalDict['LAN']['sent_message'] = message
            bge.logic.globalDict['LAN']['lastMessages'][0] = message

def initGame(game='', data=''):
    """
    Lance, modifie ou vérifie l'état d'un jeu.
    Paramètres :
        game : nom du jeu
        data : données liées au jeu
               par exemple : nom du loup, nom du zombie, position du trésor, ...
               data dépend donc du jeu
    Si on lance un nouveau jeu, game doit être indiqué (et data sera ignoré).
    Si on modifie le jeu (changement de loup, nouveau zombie, nouvelle position du trésor, ...), 
        data doit être indiqué.
    Sinon (appel sans paramètres), on vérifie l'état du jeu :
        pour loup ou zombies, on vérifie que data ne s'est pas déconnecté.
    Si besoin (nouveau jeu, nouveau loup, ...) on envoie un message.
    """

    def doClientsList():
        """
        retourne la liste des clients.
        """
        clients = []
        for client in bge.logic.globalDict['TEMP']['clients']['blue']:
            clients.append(client)
        for client in bge.logic.globalDict['TEMP']['clients']['red']:
            clients.append(client)
        return clients

    def chooseWolf(clients):
        """
        une fonction pour choisir un loup parmis les joueurs.
        S'il n'y a pas de joueur, on retourne ###.
        """
        if len(clients) > 0:
            return random.choice(clients)
        else:
            return '###'

    def choosePosition(game):
        """
        une fonction pour choisir une position
        pour le trésor ou la cible.
        """
        positionText = ''
        position = (0, 0, 0)
        if game == 'race':
            position = random.choice(utils.ciblePositions)
        elif game == 'treasure':
            position = random.choice(utils.tresorPositions)
        positionText = '{0}#{1}#{2}'.format(position[0], position[1], position[2])
        return positionText

    message = ''
    if game != '':
        # nouveau jeu :
        message = 'GAMEø{0}ø{1}ø'
        bge.logic.globalDict['TEMP']['GAME'] = game
        data = ''
        if game in ('wolf', 'zombies'):
            clients = doClientsList()
            data = chooseWolf(clients)
        elif game in ('race', 'treasure'):
            data = choosePosition(game)
        bge.logic.globalDict['TEMP']['GAME_DATA'] = data
    elif data != '':
        # même jeu ; nouveau loup :
        message = 'GAMEø{0}ø{1}ø'
        game = bge.logic.globalDict['TEMP']['GAME']
        # on vérifie l'existence du loup :
        clients = doClientsList()
        if not(data in clients):
            if game == 'wolf':
                data = chooseWolf(clients)
            elif game == 'zombies':
                if len(clients) > 0:
                    data = ''
                else:
                    data = '###'
            else:
                data = ''
        bge.logic.globalDict['TEMP']['GAME_DATA'] = data
    else:
        # même jeu ; même loup :
        game = bge.logic.globalDict['TEMP']['GAME']
        data = bge.logic.globalDict['TEMP']['GAME_DATA']
        if game in ('wolf', 'zombies'):
            # on vérifie l'existence du loup :
            clients = doClientsList()
            if not(data in clients):
                message = 'GAMEø{0}ø{1}ø'
                if game == 'wolf':
                    data = chooseWolf(clients)
                elif game == 'zombies':
                    if len(clients) > 0:
                        data = ''
                    else:
                        data = '###'
                else:
                    data = ''
                bge.logic.globalDict['TEMP']['GAME_DATA'] = data

    # on averti des changements :
    if message != '':
        message = message.format(game, data)
        sendMessage(message)



# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    state = bge.logic.globalDict['GAME'].get('state', '')
    if not(state in ('server_first', 'server_loop', 'server_quit')):
        bge.logic.globalDict['GAME']['state'] = 'server_first'

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] == 'server_first':
        # valeurs initiales (lues depuis le fichier de config si existe) :
        utils.loadSettings()
        lang = bge.logic.globalDict['MENU']['lang']
        global controller, scene
        controller = bge.logic.getCurrentController()
        scene = bge.logic.getCurrentScene()

        # affichage des textes et création du menu :
        scene.post_draw += [postDrawTexts]
        bge.logic.globalDict['TEXTS'] = {}
        bge.logic.globalDict['TEMP']['menuItems'] = {}
        bge.logic.globalDict['TEMP']['menuOver'] = ''
        # le titre :
        utils.setText(
            'progName', text=utils.PROGNAME,
            color=utils.colors['orange'], width=2, position=(0.1, 0.1))
        text = utils.tr('other', 'ServerConfig', lang)
        utils.setText(
            'title', text=text,
            color=utils.colors['white'], width=3, position=(0.3, 0.1))
        # adresse IP :
        text = '{0} :'.format(utils.tr('other', 'IP Adress', lang))
        utils.setText(
            'IP_label', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.2))
        host = findMyIp()
        utils.setText(
            'IP', text=host, 
            color=utils.colors['white'], width=2, position=(0.4, 0.2))
        # choix de la météo :
        text = '{0} :'.format(utils.tr('other', 'Meteo', lang))
        utils.setText(
            'weather_label', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.3))
        text = utils.tr('GAME_LAN', utils.weatherDict['LIST'][0], lang)
        utils.createMenuItem(
            'item_21', 
            name='WEATHER', 
            text=text,
            width=2, 
            position=(0.4, 0.3), 
            what=21)
        utils.setText('item_21', color=utils.colors['white'])
        utils.createMenuItem(
            'item_22', 
            name='WEATHER_SEND', 
            text=utils.tr('other', 'send', lang),
            width=2, 
            position=(0.8, 0.3), 
            what=22)
        # choix du jeu :
        text = '{0} :'.format(utils.tr('other', 'Game', lang))
        utils.setText(
            'game_label', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.4))
        text = utils.tr('GAME_LAN', utils.lanGames[0], lang)
        utils.createMenuItem(
            'item_31', 
            name='GAME', 
            text=text,
            width=2, 
            position=(0.4, 0.4), 
            what=31)
        utils.setText('item_31', color=utils.colors['white'])
        utils.createMenuItem(
            'item_32', 
            name='GAME_SEND', 
            text=utils.tr('other', 'send', lang),
            width=2, 
            position=(0.8, 0.4), 
            what=32)
        # écriture et envoi d'un message :
        text = '{0} :'.format(utils.tr('other', 'Message', lang))
        utils.setText(
            'message_label', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.5))
        utils.createMenuItem(
            'item_41', 
            name='MESSAGE', 
            text=utils.tr('other', 'write', lang),
            width=2, 
            position=(0.6, 0.5), 
            what=41)
        #utils.setText('item_41', color=utils.colors['white'])
        utils.createMenuItem(
            'item_42', 
            name='MESSAGE_SEND', 
            text=utils.tr('other', 'send', lang),
            width=2, 
            position=(0.8, 0.5), 
            what=42)
        # nombre de joueurs :
        text = '{0} :'.format(utils.tr('other', 'Players', lang))
        utils.setText(
            'players_label', text=text, 
            color=utils.colors['blue'], width=2, position=(0.15, 0.6))
        text = '0'
        utils.setText(
            'PLAYERS', text=text, 
            color=utils.colors['white'], width=2, position=(0.4, 0.6))
        # Démarrer/Arrêter le serveur :
        utils.createMenuItem(
            'item_81', 
            name='START', 
            text=utils.tr('other', 'START', lang),
            position=(0.2, 0.78), 
            what=81)
        utils.createMenuItem(
            'item_82', 
            name='STOP', 
            text=utils.tr('other', 'STOP', lang),
            position=(0.6, 0.78), 
            what=82, 
            ok=False)
        text = utils.tr('other', 'ServerIsStopped', lang)
        utils.setText(
            'serverState', text=text,
            color=utils.colors['white'], width=2, position=(0.3, 0.85))
        # menu retour :
        utils.createMenuItem(
            'item_91', 
            name='MENU', 
            text=utils.tr('menu', 'MENU', lang),
            position=(0.1, 0.95), 
            what=91)

        # on affiche la souris :
        bge.logic.mouse.position = bge.logic.globalDict['GAME']['cursorPosition']
        bge.render.showMouse(True)
        # paramètres des échanges réseau :
        bge.logic.globalDict['LAN']['sent_message'] = ''
        bge.logic.globalDict['LAN']['received_message'] = ''
        bge.logic.globalDict['LAN']['lastMessages'] = ['', '']
        bge.logic.globalDict['LAN']['host'] = host
        bge.logic.globalDict['LAN']['user'] = 'SERVER'
        # initialisation terminée :
        bge.logic.globalDict['TEMP']['initialTime'] = int(time.time())
        bge.logic.globalDict['TEMP']['actualTime'] = 0
        bge.logic.globalDict['TEMP']['clientsText'] = ''
        bge.logic.globalDict['TEMP']['clients'] = {'blue':[], 'red':[]}
        bge.logic.globalDict['TEMP']['WEATHER'] = utils.weatherDict['LIST'][0]
        bge.logic.globalDict['TEMP']['GAME'] = utils.lanGames[0]
        bge.logic.globalDict['TEMP']['GAME_DATA'] = ''
        bge.logic.globalDict['TEMP']['SERVER_MESSAGE'] = ''
        bge.logic.globalDict['GAME']['state'] = 'server_loop'

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] != 'server_loop':
        return
    if bge.logic.globalDict['GAME']['pause']:
        return
    # récupération des coordonnées de la souris :
    cursorPosition = input_controller.getCursorPosition()
    # calculs divers concernant les durées :
    actualTime = int(time.time() - bge.logic.globalDict['TEMP']['initialTime'])
    # affichage des menus et récupération de l'action active :
    what = -1
    for menuItemName in bge.logic.globalDict['TEMP']['menuItems']:
        menuItem = bge.logic.globalDict['TEMP']['menuItems'][menuItemName]
        if menuItem['ok']:
            menuPosition = menuItem['position']
            over = input_controller.mouseOver(cursorPosition, menuPosition)
            if over:
                what = menuItem['what']
                lang = bge.logic.globalDict['MENU']['lang']
                if menuItemName != bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = menuItemName
                    menuItem['width'] = menuItem['initialWidth'] * 1.5
                    utils.setText(menuItemName, width=menuItem['width'])
                    utils.playSound('over')
            else:
                menuItem['width'] = menuItem['initialWidth']
                utils.setText(menuItemName, width=menuItem['width'])
                if menuItemName == bge.logic.globalDict['TEMP']['menuOver']:
                    bge.logic.globalDict['TEMP']['menuOver'] = ''
    inputState = input_controller.inputManagement('menu')
    # pour faire un screencast :
    if inputState['SCREENSHOT']:
        utils.makeScreenshot()
    # gestion des menus et autres actions :
    if (what > -1) and (inputState['ENTER']):
        utils.playSound('clic')
        doAction(what)
        return
    elif inputState['QUIT']:
        doAction(91)
        return
    elif inputState['LOCALHOST']:
        # passe en localhost :
        utils.setText('IP', text='localhost')
        bge.logic.globalDict['LAN']['host'] = 'localhost'

    # message reçu :
    received_message = bge.logic.globalDict['LAN']['received_message']
    if not(received_message in ('', bge.logic.globalDict['LAN']['lastMessages'][1])):
        try:
            received_message = received_message.split('&')
            if len(received_message) > 1:
                newClientsText = received_message[1][8:]
                if newClientsText != bge.logic.globalDict['TEMP']['clientsText']:
                    bge.logic.globalDict['TEMP']['clientsText'] = newClientsText
                    bge.logic.globalDict['TEMP']['clients'] = {'blue':[], 'red':[]}
                    clients = newClientsText.split('ø')
                    for client in clients:
                        clientDatas = client.split('#')
                        if len(clientDatas) > 2:
                            if clientDatas[2] == 'blue':
                                bge.logic.globalDict['TEMP']['clients']['blue'].append(clientDatas[1])
                            elif clientDatas[2] == 'red':
                                bge.logic.globalDict['TEMP']['clients']['red'].append(clientDatas[1])
                    blue = len(bge.logic.globalDict['TEMP']['clients']['blue'])
                    red = len(bge.logic.globalDict['TEMP']['clients']['red'])
                    text = '{0}    ({1}, {2})'.format(blue + red, blue, red)
                    utils.setText('PLAYERS', text=text)
                    initGame()
            received_message = received_message[0].split('ø')
            threadName = received_message[0]
            who = received_message[1]
            what = received_message[2]
            if what == 'DISCONNECT':
                clientsText = bge.logic.globalDict['TEMP']['clientsText']
                for color in ('blue', 'red'):
                    if who in bge.logic.globalDict['TEMP']['clients'][color]:
                        bge.logic.globalDict['TEMP']['clients'][color].remove(who)
                        clientsText = clientsText.replace('{0}#{1}ø'.format(who, color), '')
                bge.logic.globalDict['TEMP']['clientsText'] = clientsText
                blue = len(bge.logic.globalDict['TEMP']['clients']['blue'])
                red = len(bge.logic.globalDict['TEMP']['clients']['red'])
                text = '{0}    ({1}, {2})'.format(blue + red, blue, red)
                utils.setText('PLAYERS', text=text)
                initGame()
            elif what == 'COLLISION':
                initGame(data=received_message[6])
            bge.logic.globalDict['LAN']['lastMessages'][1] = received_message
        except:
            #print('ERREUR received_message :', received_message)
            pass


    # la suite ne sera effectuée que toutes les secondes :
    if actualTime <= bge.logic.globalDict['TEMP']['actualTime']:
        return
    # mise à jour du actualTime :
    bge.logic.globalDict['TEMP']['actualTime'] = actualTime

    """
    # la suite ne sera effectuée que toutes les 5 secondes :
    if (actualTime % 5) > 0:
        return
    """

    # on renvoie la météo et le jeu pour les nouveaux joueurs :
    message = 'LOOPøSERVERøNONEøNONEøWEATHER:{0}øGAME:{1}øDATA:{2}øBLUE:{3}øRED:{4}'.format(
        bge.logic.globalDict['TEMP']['WEATHER'],
        bge.logic.globalDict['TEMP']['GAME'],
        bge.logic.globalDict['TEMP']['GAME_DATA'],
        len(bge.logic.globalDict['TEMP']['clients']['blue']),
        len(bge.logic.globalDict['TEMP']['clients']['red']))
    sendMessage(message, force=True)


